/**********************************************************************
 * $Id: OGRSpatialReference.java,v 1.3 2006/01/09 12:50:13 nacho Exp $
 *
 * Name:     OGRSpatialReference.java
 * Project:  JGDAL. Interface java to gdal (Frank Warmerdam).
 * Purpose:   
 * Author:   Nacho Brodin, brodin_ign@gva.es
 *
 **********************************************************************/
/*Copyright (C) 2004  Nacho Brodin <brodin_ign@gva.es>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package es.gva.cit.jogr;


import java.io.*;
import java.util.Date;

//import es.gva.cit.wrappergdal.OGRException;

/** 
 * Representa un sistema de referencia espacial OpenGIS.
 * 
 * @author Nacho Brodin <brodin_ign@gva.es>.<BR> Equipo de desarrollo gvSIG.<BR> http://www.gvsig.gva.es
 * @version 0.0
 * @link http://www.gvsig.gva.es
 */

public class OGRSpatialReference extends JNIBase{
	
	private native int setUTMNat(long cPtr, int zona, int norte_sur);
	private native int setWellKnownGeogCSNat(long cPtr, String cs);
	private native long OGRSpatialReferenceNat();
	private native String exportToWktNat(long cPtr);
	private native void OSRDestroySpatialReferenceNat(long cPtr);
	private native int getUTMZoneNat(long cPtr);
	private native int getHemisphereNat(long cPtr);
	private native int setFromUserInputNat(long cPtr, String a_srs);
	
	/**
	 *Constructor a partir de la direccin de memoria 
	 */
	
	public OGRSpatialReference(long cPtr){
		this.cPtr=cPtr;
	}
	
	/**
	 * Constructor generico 
	 */
	
	public OGRSpatialReference(){
		cPtr = OGRSpatialReferenceNat();
	}
	
	/**
	 * Escribe la zona UTM pasada por parmetro
	 * @param zona	Zona UTM
	 * @param norte_sur	TRUE hemisferio norte y FALSE hemisferio sur
	 * @throws OGRException
	 */
	
	public void setUTM(int zona, boolean norte_sur)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setUTM(). La llamada de creacin de objeto no tuvo exito");
		
		int ns=-1;
		if(norte_sur)ns=1;
		else ns=0;
		
		int res = setUTMNat(cPtr, zona, ns);
		
		if(res<0)
			throw new OGRException("Error en setUTM(). No se ha podido asignar la zona especificada.");
	}
	
	/**
	 * Lee la zona UTM 
	 * @return Zona UTM
	 * @throws OGRException
	 */
	
	public int getUTMZone()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en getUTMZone(). La llamada de creacin de objeto no tuvo exito");
		
		int res = getUTMZoneNat(cPtr);
		
		if(res<0)
			throw new OGRException("Error en getUTMZone(). Valor de zona devuelto erroneo.");
		
		return res;	
	}
	
	/**
	 * Obtiene el hemisferio
	 * @return TRUE hemisferio norte y FALSE hemisferio sur
	 * @throws OGRException
	 */
	
	public boolean getHemisphere()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en getHemisphere(). La llamada de creacin de objeto no tuvo exito");
		
		int hemis = getHemisphereNat(cPtr);
		 			
		if(hemis<0)
			throw new OGRException("Error en getHemisphere(). Valor de hemisferio devuelto erroneo.");
		
		if(hemis==1)return true;
		else return false;
	}
	
	/**
	 *Asigna las coordenadas geogrficas
	 *@param cs	Coordenadas geograficas soportadas:<BR>
	 *<UL>
	 *<LI>WGS84</LI>
	 *<LI>WGS72</LI>
	 *<LI>NAD27</LI>
	 *<LI>NAD83</LI>
	 *<LI>EPSG:n</LI>
	 *</UL>
	 *@throws OGRException
	 */
	
	public void setWellKnownGeogCS( String cs)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setWellKnownGeogCS(). La llamada de creacin de objeto no tuvo exito");
		
		int res = setWellKnownGeogCSNat(cPtr, cs);
		
		if(res<0)
			throw new OGRException("Error en setWellKnownGeoCS(). No se ha podido asignar el sistema de coordenadas especificado.");
	}
	
	/**
	 * Devuelve la codificacin WKT
	 * @return Codificacin WKT
	 * @throws OGRException
	 */
	public String exportToWkt()throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en exportToWkt(). La llamada de creacin de objeto no tuvo exito");
		
		String wkt = exportToWktNat(cPtr);
		
		if(wkt==null)
			throw new OGRException("Error en exportToWkt(). No se ha podido obtener la proyeccin.");
		
		return wkt;
	}
	
	
	/**
	 * @throws OGRException
	 * @param a_srs	
	 */
	
	public void setFromUserInput(String a_srs)throws OGRException{
		
		if(cPtr <= 0)
			throw new OGRException("Error en setFromUserInput(). La llamada de creacin de objeto no tuvo exito");
		
		int ogrerr = setFromUserInputNat(cPtr,a_srs);
		lanzarExcepcion(ogrerr,"Error en setFromUserInput().");

	}
	
	
	/**
	 * Destructor 
	 */
	
	protected void finalize(){
		if(cPtr > 0)
			OSRDestroySpatialReferenceNat(cPtr);
	}
	
}